import pandas as pd
import FinanceDataReader as fdr
import seaborn as sns
import mplfinance as mpf

from tqdm import tqdm

colorset = mpf.make_marketcolors(up='tab:red', down='tab:blue', volume='tab:blue')
mpf_style = mpf.make_mpf_style(marketcolors=colorset)

def get_buy_1t(df, t):
    """
    df = dataframe
    t: 매수 시기
    하나의 종목을 백테스트
    df, index,Close,언제 매수할 지 필요
    :return:
    """

    # 매수 이후 언제까지 수익률을 확인할지
    days_back = 30
    initial_cash = 10000000

    # 인덱스 위치 구하기
    df1 = df.reset_index(drop=False)
    index_no = df1[df1.Date == t].index.values[0]

    df2 = df.iloc[index_no:index_no+days_back]
    df2['days'] = df2.reset_index(drop=False).index
    df2['days'] = df2['days'] + 1

    df2 = df2[['Close', 'days']]
    df2['valamt'] = 0
    df2['holdamt'] = 0
    df2['holdcnt'] = None
    df2['cash'] = None

    import math

    df2['valamt'].iloc[0] = initial_cash
    df2['holdcnt'].iloc[0] = math.floor(initial_cash / df2['Close'].iloc[0])
    df2['holdcnt'] = df2['holdcnt'].fillna(method = 'ffill')

    df2['holdamt'].iloc[0] = df2['Close'].iloc[0] * df2['holdcnt'].iloc[0]
    df2['cash'].iloc[0] = df2['valamt'].iloc[0] - df2['holdamt'].iloc[0]
    df2['cash'] = df2['cash'].fillna(method='ffill')

    df2['holdamt'].iloc[1:] = df2['holdcnt'][1:] * df2['Close'][1:]
    df2['valamt'][1:] = df2['holdamt'][1:] + df2['cash'][1:]
    df2['yrate'] = df2['valamt'] / initial_cash *100

    return df2


def backtest(df, col, ticker):
    """
    매수 여부에 따라 평가 금액을 계산한다. get_buy_1d로 매수
    :param df: 데이터프레임 매수여부가 있어야 함
    :param col: 매수여부가 있는 칼럼
    :param ticker: 종목코드 기록용
    :return:
    """
    import matplotlib.pyplot as plt

    temp = df.query(f'{col}==1')
    temp.index.to_list()

    tot = pd.DataFrame()
    for t in tqdm(temp.index.to_list()):
        b1 = get_buy_1t(df,t)
        b1['ticker'] = ticker
        tot = pd.concat([b1, tot], axis=0)

    temp = tot['days'].value_counts()
    sns.countplot(data=tot, x='days').set(title='days after signal')
    plt.show()

    tot_gr = tot.groupby('days')['yrate'].mean()
    sns.lineplot(x=tot_gr.index, y=tot_gr).set(title='yield rate by closed price')
    plt.show()

    tot['updown'] = tot['yrate'].apply(lambda x: 'up' if x >= 100 else 'down')
    sns.countplot(data=tot, x='days', hue='updown').set(title='up down cnt')
    plt.show()

    return tot

